'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sequenceEventActions = undefined;

var _identity = require('lodash/identity');

var _identity2 = _interopRequireDefault(_identity);

var _capitalize = require('lodash/capitalize');

var _capitalize2 = _interopRequireDefault(_capitalize);

var _partial = require('lodash/partial');

var _partial2 = _interopRequireDefault(_partial);

var _mapValues = require('lodash/mapValues');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _compose = require('lodash/fp/compose');

var _compose2 = _interopRequireDefault(_compose);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

var _icepick = require('icepick');

var _icepick2 = _interopRequireDefault(_icepick);

var _index = require('./index');

var _actions2 = require('../actions');

var _actions3 = _interopRequireDefault(_actions2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var asyncSetValidity = _actions3.default.asyncSetValidity;
var blur = _actions3.default.blur;
var focus = _actions3.default.focus;
var setValidity = _actions3.default.setValidity;


function persistEventWithCallback(callback) {
  return function (event) {
    if (event && event.persist) {
      event.persist();
    }

    callback(event);
    return event;
  };
}

var modelValueUpdaterMap = {
  checkbox: function checkbox(props, eventValue) {
    var model = props.model;
    var modelValue = props.modelValue;


    if ((0, _index.isMulti)(model)) {
      var valueWithItem = modelValue || [];
      var valueWithoutItem = (valueWithItem || []).filter(function (item) {
        return !(0, _isEqual2.default)(item, eventValue);
      });
      var value = valueWithoutItem.length === valueWithItem.length ? _icepick2.default.push(valueWithItem, eventValue) : valueWithoutItem;

      return value;
    }

    return !modelValue;
  },
  default: function _default(props, eventValue) {
    return eventValue;
  }
};

function isReadOnlyValue(control) {
  return control.type === 'input' // verify === is okay
   && ~['radio', 'checkbox'].indexOf(control.props.type);
}

function deprecateUpdateOn(updateOn) {
  console.warn('Using the updateOn prop as a function will be deprecated in v1.0. ' + 'Please use the changeAction prop instead.');

  return updateOn;
}

function sequenceEventActions(control, props) {
  var dispatch = props.dispatch;
  var model = props.model;
  var updateOn = props.updateOn;
  var parser = props.parser;
  var changeAction = props.changeAction;
  var _control$props = control.props;
  var _control$props$onChan = _control$props.onChange;
  var onChange = _control$props$onChan === undefined ? _identity2.default : _control$props$onChan;
  var _control$props$onBlur = _control$props.onBlur;
  var onBlur = _control$props$onBlur === undefined ? _identity2.default : _control$props$onBlur;
  var _control$props$onFocu = _control$props.onFocus;
  var onFocus = _control$props$onFocu === undefined ? _identity2.default : _control$props$onFocu;


  var controlOnChange = persistEventWithCallback(onChange);
  var controlOnBlur = persistEventWithCallback(onBlur);
  var controlOnFocus = persistEventWithCallback(onFocus);

  var updateOnEventHandler = typeof updateOn === 'function' ? 'onChange' : 'on' + (0, _capitalize2.default)(props.updateOn);
  var validateOn = 'on' + (0, _capitalize2.default)(props.validateOn);
  var asyncValidateOn = 'on' + (0, _capitalize2.default)(props.asyncValidateOn);

  var updaterFn = typeof updateOn === 'function' ? deprecateUpdateOn(updateOn) : _identity2.default;

  var eventActions = {
    onFocus: [function () {
      return dispatch(focus(model));
    }],
    onBlur: [function () {
      return dispatch(blur(model));
    }],
    onChange: [],
    onLoad: [], // pseudo-event
    onSubmit: [] };

  // pseudo-event
  var controlChangeMethod = (0, _partial2.default)(changeAction, model);
  var modelValueUpdater = modelValueUpdaterMap[control.props.type] || modelValueUpdaterMap.default;

  if (control.props.defaultValue) {
    eventActions.onLoad.push(function () {
      return dispatch(_actions3.default.change(model, control.props.defaultValue));
    });
  }

  if (props.validators || props.errors) {
    var dispatchValidate = function dispatchValidate(value) {
      if (props.validators) {
        dispatch(setValidity(model, (0, _index.getValidity)(props.validators, value)));
      }

      if (props.errors) {
        dispatch(setValidity(model, (0, _index.getValidity)(props.errors, value), {
          errors: true
        }));
      }

      return value;
    };

    eventActions[validateOn].push(dispatchValidate);
    eventActions.onLoad.push(dispatchValidate);
  }

  if (props.asyncValidators) {
    var dispatchAsyncValidate = function dispatchAsyncValidate(value) {
      (0, _mapValues2.default)(props.asyncValidators, function (validator, key) {
        return dispatch(asyncSetValidity(model, function (_, done) {
          var outerDone = function outerDone(valid) {
            return done(_defineProperty({}, key, valid));
          };

          validator((0, _index.getValue)(value), outerDone);
        }));
      });

      return value;
    };

    eventActions[asyncValidateOn].push(dispatchAsyncValidate);
  }

  var dispatchChange = function dispatchChange(event) {
    var modelValue = isReadOnlyValue(control) ? modelValueUpdater(props, control.props.value) : event;

    dispatch(controlChangeMethod(modelValue));

    return modelValue;
  };

  eventActions.onSubmit.push(updaterFn(dispatchChange));

  if (!isReadOnlyValue(control)) {
    eventActions[updateOnEventHandler].push((0, _compose2.default)(updaterFn(dispatchChange), (0, _partial2.default)(modelValueUpdater, props), parser, _index.getValue, controlOnChange));
  } else {
    eventActions[updateOnEventHandler].push(updaterFn(dispatchChange));
  }
  eventActions.onBlur.push(controlOnBlur);
  eventActions.onFocus.push(controlOnFocus);

  return (0, _mapValues2.default)(eventActions, function (_actions) {
    return _compose2.default.apply(undefined, _toConsumableArray(_actions));
  });
}

exports.sequenceEventActions = sequenceEventActions;